@@ -20,6 +20,7 @@ import java.util.Collections;
 import java.util.Date;
 import java.util.List;
 import java.util.Objects;
+import java.util.Optional;
 import java.util.stream.Collectors;
 import java.util.stream.StreamSupport;
 
@@ -29,6 +30,8 @@ import javax.enterprise.event.Event;
 
 import com.google.gwt.event.shared.EventBus;
 import com.google.gwt.event.shared.HandlerRegistration;
+import org.drools.scenariosimulation.api.model.FactMappingValueType;
+import org.drools.workbench.screens.scenariosimulation.client.commands.actualcommands.AbstractScenarioGridCommand;
 import org.drools.workbench.screens.scenariosimulation.client.commands.actualcommands.AbstractScenarioSimulationCommand;
 import org.drools.workbench.screens.scenariosimulation.client.commands.actualcommands.AppendColumnCommand;
 import org.drools.workbench.screens.scenariosimulation.client.commands.actualcommands.AppendRowCommand;
@@ -50,6 +53,7 @@ import org.drools.workbench.screens.scenariosimulation.client.commands.actualcom
 import org.drools.workbench.screens.scenariosimulation.client.commands.actualcommands.SetInstanceHeaderCommand;
 import org.drools.workbench.screens.scenariosimulation.client.commands.actualcommands.SetPropertyHeaderCommand;
 import org.drools.workbench.screens.scenariosimulation.client.editor.ScenarioSimulationEditorPresenter;
+import org.drools.workbench.screens.scenariosimulation.client.enums.GridWidget;
 import org.drools.workbench.screens.scenariosimulation.client.events.AppendColumnEvent;
 import org.drools.workbench.screens.scenariosimulation.client.events.AppendRowEvent;
 import org.drools.workbench.screens.scenariosimulation.client.events.DeleteColumnEvent;
@@ -74,6 +78,7 @@ import org.drools.workbench.screens.scenariosimulation.client.events.SetInstance
 import org.drools.workbench.screens.scenariosimulation.client.events.SetPropertyHeaderEvent;
 import org.drools.workbench.screens.scenariosimulation.client.events.UndoEvent;
 import org.drools.workbench.screens.scenariosimulation.client.events.UnsupportedDMNEvent;
+import org.drools.workbench.screens.scenariosimulation.client.events.ValidateSimulationEvent;
 import org.drools.workbench.screens.scenariosimulation.client.handlers.AppendColumnEventHandler;
 import org.drools.workbench.screens.scenariosimulation.client.handlers.AppendRowEventHandler;
 import org.drools.workbench.screens.scenariosimulation.client.handlers.DeleteColumnEventHandler;
@@ -98,11 +103,14 @@ import org.drools.workbench.screens.scenariosimulation.client.handlers.SetInstan
 import org.drools.workbench.screens.scenariosimulation.client.handlers.SetPropertyHeaderEventHandler;
 import org.drools.workbench.screens.scenariosimulation.client.handlers.UndoEventHandler;
 import org.drools.workbench.screens.scenariosimulation.client.handlers.UnsupportedDMNEventHandler;
+import org.drools.workbench.screens.scenariosimulation.client.handlers.ValidateSimulationEventHandler;
+import org.drools.workbench.screens.scenariosimulation.client.models.AbstractScesimGridModel;
 import org.drools.workbench.screens.scenariosimulation.client.popup.ConfirmPopupPresenter;
 import org.drools.workbench.screens.scenariosimulation.client.popup.DeletePopupPresenter;
 import org.drools.workbench.screens.scenariosimulation.client.popup.FileUploadPopupPresenter;
 import org.drools.workbench.screens.scenariosimulation.client.popup.PreserveDeletePopupPresenter;
 import org.drools.workbench.screens.scenariosimulation.client.resources.i18n.ScenarioSimulationEditorConstants;
+import org.drools.workbench.screens.scenariosimulation.client.utils.ConstantHolder;
 import org.drools.workbench.screens.scenariosimulation.client.widgets.ScenarioGridColumn;
 import org.kie.workbench.common.command.client.CommandResult;
 import org.kie.workbench.common.command.client.CommandResultBuilder;
@@ -139,7 +147,8 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
                                                        SetInstanceHeaderEventHandler,
                                                        SetPropertyHeaderEventHandler,
                                                        UndoEventHandler,
-                                                       UnsupportedDMNEventHandler {
+                                                       UnsupportedDMNEventHandler,
+                                                       ValidateSimulationEventHandler {
 
     protected DeletePopupPresenter deletePopupPresenter;
     protected PreserveDeletePopupPresenter preserveDeletePopupPresenter;
@@ -154,10 +163,12 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
 
     protected ScenarioSimulationEditorPresenter scenarioSimulationEditorPresenter;
 
-    protected ScenarioCommandRegistry scenarioCommandRegistry;
+    protected ScenarioCommandRegistryManager scenarioCommandRegistryManager;
 
     protected ScenarioCommandManager scenarioCommandManager;
 
+    protected ScenarioSimulationContext context;
+
     public void setEventBus(EventBus eventBus) {
         this.eventBus = eventBus;
         registerHandlers();
@@ -187,16 +198,16 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
         this.scenarioSimulationEditorPresenter = scenarioSimulationEditorPresenter;
     }
 
-    public void setScenarioCommandRegistry(ScenarioCommandRegistry scenarioCommandRegistry) {
-        this.scenarioCommandRegistry = scenarioCommandRegistry;
+    public void setScenarioCommandRegistryManager(ScenarioCommandRegistryManager scenarioCommandRegistryManager) {
+        this.scenarioCommandRegistryManager = scenarioCommandRegistryManager;
     }
 
     public void setScenarioCommandManager(ScenarioCommandManager scenarioCommandManager) {
         this.scenarioCommandManager = scenarioCommandManager;
     }
 
-    private ScenarioSimulationContext getFocusedContext() {
-        return scenarioSimulationEditorPresenter.getFocusedContext();
+    public void setContext(ScenarioSimulationContext context) {
+        this.context = context;
     }
 
     @PreDestroy
@@ -206,140 +217,105 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
 
     @Override
     public void onEvent(AppendColumnEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setColumnId(String.valueOf(new Date().getTime()));
         context.getStatus().setColumnGroup(event.getColumnGroup());
-        commonExecution(context,
-                        new AppendColumnCommand(),
-                        true);
+        commonExecution(new AppendColumnCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(AppendRowEvent event) {
-        commonExecution(getFocusedContext(),
-                        new AppendRowCommand(),
-                        true);
+        commonExecution(new AppendRowCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(DeleteColumnEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setColumnIndex(event.getColumnIndex());
         context.getStatus().setColumnGroup(event.getColumnGroup());
         context.getStatus().setDisable(true);
         context.getStatus().setOpenDock(false);
         context.getStatus().setAsProperty(event.isAsProperty());
-        commonExecution(context,
-                        new DeleteColumnCommand(),
-                        true);
+        commonExecution(new DeleteColumnCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(DeleteRowEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setRowIndex(event.getRowIndex());
-        commonExecution(context,
-                        new DeleteRowCommand(),
-                        true);
+        commonExecution(new DeleteRowCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(DisableTestToolsEvent event) {
-        commonExecution(getFocusedContext(),
-                        new DisableTestToolsCommand(),
-                        false);
+        commonExecution(new DisableTestToolsCommand(), false);
     }
 
     @Override
     public void onEvent(DuplicateInstanceEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setColumnId(String.valueOf(new Date().getTime()));
         context.getStatus().setColumnIndex(event.getColumnIndex());
         context.getStatus().setRight(true);
         context.getStatus().setAsProperty(false);
         context.getStatus().setFullPackage(
-                ((ScenarioGridColumn) context.getModel().getSelectedColumn()).getFactIdentifier().getPackageWithoutClassName());
-        commonExecution(context,
-                        new DuplicateInstanceCommand(),
-                        true);
+                ((ScenarioGridColumn) context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).getSelectedColumn()).getFactIdentifier().getPackageWithoutClassName());
+        commonExecution(new DuplicateInstanceCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(DuplicateRowEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setRowIndex(event.getRowIndex());
-        commonExecution(context,
-                        new DuplicateRowCommand(),
-                        true);
+        commonExecution(new DuplicateRowCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(EnableTestToolsEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setFilterTerm(event.getFilterTerm());
         context.getStatus().setPropertyNameElements(event.getPropertyNameElements());
         context.getStatus().setNotEqualsSearch(event.isNotEqualsSearch());
-        commonExecution(context,
-                        new EnableTestToolsCommand(),
-                        false);
+        commonExecution(new EnableTestToolsCommand(), false);
     }
 
     @Override
     public void onEvent(ImportEvent event) {
         org.uberfire.mvp.Command okImportCommand = () -> {
-            ImportCommand importCommand = new ImportCommand();
-            importCommand.setFileContent(fileUploadPopupPresenter.getFileContents());
-            commonExecution(getFocusedContext(),
-                            importCommand,
-                            false);
+            ImportCommand importCommand = new ImportCommand(event.getGridWidget(), fileUploadPopupPresenter.getFileContents());
+            commonExecution(importCommand, false);
         };
         fileUploadPopupPresenter.show(Collections.singletonList(CSV.getExtension()),
                                       ScenarioSimulationEditorConstants.INSTANCE.selectImportFile(),
+                                      ScenarioSimulationEditorConstants.INSTANCE.uploadWarning(),
                                       ScenarioSimulationEditorConstants.INSTANCE.importLabel(),
                                       okImportCommand);
     }
 
     @Override
     public void onEvent(InsertColumnEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setColumnId(String.valueOf(new Date().getTime()));
         context.getStatus().setColumnIndex(event.getColumnIndex());
         context.getStatus().setRight(event.isRight());
         context.getStatus().setAsProperty(event.isAsProperty());
-        commonExecution(context,
-                        new InsertColumnCommand(),
-                        true);
+        commonExecution(new InsertColumnCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(InsertRowEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setRowIndex(event.getRowIndex());
-        commonExecution(context,
-                        new InsertRowCommand(),
-                        true);
+        commonExecution(new InsertRowCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(PrependColumnEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setColumnId(String.valueOf(new Date().getTime()));
         context.getStatus().setColumnGroup(event.getColumnGroup());
-        commonExecution(context,
-                        new PrependColumnCommand(),
-                        true);
+        commonExecution(new PrependColumnCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(PrependRowEvent event) {
-        commonExecution(getFocusedContext(),
-                        new PrependRowCommand(),
-                        true);
+        commonExecution(new PrependRowCommand(event.getGridWidget()), true);
     }
 
     @Override
     public void onEvent(RedoEvent event) {
-        final CommandResult<ScenarioSimulationViolation> status = scenarioCommandRegistry.redo(getFocusedContext());
+        final CommandResult<ScenarioSimulationViolation> status = scenarioCommandRegistryManager.redo(context);
         if (Objects.equals(CommandResult.Type.ERROR, status.getType())) {
             commonNotifyError(status, ScenarioSimulationEditorConstants.INSTANCE.redo());
         }
@@ -347,26 +323,20 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
 
     @Override
     public void onEvent(ReloadTestToolsEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setDisable(event.isDisable());
         context.getStatus().setOpenDock(event.isOpenDock());
-        commonExecution(context,
-                        new ReloadTestToolsCommand(),
-                        false);
+        commonExecution(new ReloadTestToolsCommand(), false);
     }
 
     @Override
     public void onEvent(RunSingleScenarioEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setRowIndex(event.getRowIndex());
-        commonExecution(context,
-                        new RunSingleScenarioCommand(),
-                        false);
+        commonExecution(new RunSingleScenarioCommand(), false);
     }
 
     @Override
-    public void handle(ScenarioGridReloadEvent event) {
-        getFocusedContext().getScenarioGridPanel().onResize();
+    public void onEvent(ScenarioGridReloadEvent event) {
+        context.getScenarioGridPanelByGridWidget(event.getGridWidget()).onResize();
     }
 
     @Override
@@ -376,37 +346,29 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
 
     @Override
     public void onEvent(SetGridCellValueEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setRowIndex(event.getRowIndex());
         context.getStatus().setColumnIndex(event.getColumnIndex());
         context.getStatus().setGridCellValue(event.getCellValue());
-        commonExecution(context,
-                        new SetGridCellValueCommand(),
-                        false);
+        commonExecution(new SetGridCellValueCommand(event.getGridWidget()), false);
     }
 
     @Override
     public void onEvent(SetHeaderCellValueEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
         context.getStatus().setRowIndex(event.getRowIndex());
         context.getStatus().setColumnIndex(event.getColumnIndex());
         context.getStatus().setHeaderCellValue(event.getHeaderCellValue());
-        commonExecution(context,
-                        new SetHeaderCellValueCommand(event.isInstanceHeader(), event.isPropertyHeader()),
-                        false);
+        commonExecution(new SetHeaderCellValueCommand(event.getGridWidget(), event.isInstanceHeader(), event.isPropertyHeader()), false);
     }
 
     @Override
     public void onEvent(SetInstanceHeaderEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
-        if (context.getModel().isSameInstanceType(event.getClassName())) {
+        if (context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).isSameInstanceType(event.getClassName())) {
             return;
         }
         context.getStatus().setFullPackage(event.getFullPackage());
         context.getStatus().setClassName(event.getClassName());
-        if (((ScenarioGridColumn) context.getModel().getSelectedColumn()).isInstanceAssigned()) {
-            org.uberfire.mvp.Command okPreserveCommand = () -> commonExecution(context,
-                                                                               new SetInstanceHeaderCommand(),
+        if (((ScenarioGridColumn) context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).getSelectedColumn()).isInstanceAssigned()) {
+            org.uberfire.mvp.Command okPreserveCommand = () -> commonExecution(new SetInstanceHeaderCommand(event.getGridWidget()),
                                                                                true);
             deletePopupPresenter.show(ScenarioSimulationEditorConstants.INSTANCE.changeTypeMainTitle(),
                                       ScenarioSimulationEditorConstants.INSTANCE.changeTypeMainQuestion(),
@@ -416,44 +378,40 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
                                       ScenarioSimulationEditorConstants.INSTANCE.changeType(),
                                       okPreserveCommand);
         } else {
-            commonExecution(context,
-                            new SetInstanceHeaderCommand(),
-                            true);
+            commonExecution(new SetInstanceHeaderCommand(event.getGridWidget()), true);
         }
     }
 
     @Override
     public void onEvent(SetPropertyHeaderEvent event) {
-        ScenarioSimulationContext context = getFocusedContext();
-        if (context.getModel().getSelectedColumn() == null) {
+        if (context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).getSelectedColumn() == null) {
             return;
         }
-        if (context.getModel().isAlreadyAssignedProperty(event.getPropertyNameElements())) {
-            String value = String.join(".", event.getPropertyNameElements());
+        if (context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).isAlreadyAssignedProperty(event.getPropertyNameElements())) {
+            String value;
+            if (Objects.equals(FactMappingValueType.EXPRESSION, event.getFactMappingValueType())) {
+                value = ConstantHolder.EXPRESSION;
+            } else {
+                value = String.join(".", event.getPropertyNameElements());
+            }
             onEvent(new ScenarioNotificationEvent("Property \"" + value + "\" already assigned", NotificationEvent.NotificationType.ERROR));
             return;
         }
         context.getStatus().setFullPackage(event.getFullPackage());
         context.getStatus().setPropertyNameElements(event.getPropertyNameElements());
         context.getStatus().setValueClassName(event.getValueClassName());
-        if (context.getModel().isSelectedColumnEmpty()) {
-            commonExecution(context,
-                            new SetPropertyHeaderCommand(),
-                            true);
-        } else if (context.getModel().isSameSelectedColumnProperty(event.getPropertyNameElements())) {
+        if (context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).isSelectedColumnEmpty()) {
+            commonExecution(new SetPropertyHeaderCommand(event.getGridWidget(), event.getFactMappingValueType()), true);
+        } else if (context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).isSameSelectedColumnProperty(event.getPropertyNameElements())) {
             return;
-        } else if (context.getModel().isSameSelectedColumnType(event.getValueClassName())) {
+        } else if (context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).isSameSelectedColumnType(event.getValueClassName())) {
             org.uberfire.mvp.Command okDeleteCommand = () -> {
                 context.getStatus().setKeepData(false);
-                commonExecution(context,
-                                new SetPropertyHeaderCommand(),
-                                true);
+                commonExecution(new SetPropertyHeaderCommand(event.getGridWidget(), event.getFactMappingValueType()), true);
             };
             org.uberfire.mvp.Command okPreserveCommand = () -> {
                 context.getStatus().setKeepData(true);
-                commonExecution(context,
-                                new SetPropertyHeaderCommand(),
-                                true);
+                commonExecution(new SetPropertyHeaderCommand(event.getGridWidget(), event.getFactMappingValueType()), true);
             };
             preserveDeletePopupPresenter.show(ScenarioSimulationEditorConstants.INSTANCE.preserveDeleteScenarioMainTitle(),
                                               ScenarioSimulationEditorConstants.INSTANCE.preserveDeleteScenarioMainQuestion(),
@@ -465,12 +423,10 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
                                               ScenarioSimulationEditorConstants.INSTANCE.deleteValues(),
                                               okPreserveCommand,
                                               okDeleteCommand);
-        } else if (!context.getModel().isSameSelectedColumnType(event.getValueClassName())) {
+        } else if (!context.getAbstractScesimGridModelByGridWidget(event.getGridWidget()).isSameSelectedColumnType(event.getValueClassName())) {
             org.uberfire.mvp.Command okPreserveCommand = () -> {
                 context.getStatus().setKeepData(false);
-                commonExecution(context,
-                                new SetPropertyHeaderCommand(),
-                                true);
+                commonExecution(new SetPropertyHeaderCommand(event.getGridWidget(), event.getFactMappingValueType()), true);
             };
             deletePopupPresenter.show(ScenarioSimulationEditorConstants.INSTANCE.deleteScenarioMainTitle(),
                                       ScenarioSimulationEditorConstants.INSTANCE.deleteScenarioMainQuestion(),
@@ -484,7 +440,7 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
 
     @Override
     public void onEvent(UndoEvent event) {
-        final CommandResult<ScenarioSimulationViolation> status = scenarioCommandRegistry.undo(getFocusedContext());
+        final CommandResult<ScenarioSimulationViolation> status = scenarioCommandRegistryManager.undo(context);
         if (Objects.equals(CommandResult.Type.ERROR, status.getType())) {
             commonNotifyError(status, ScenarioSimulationEditorConstants.INSTANCE.undo());
         }
@@ -495,16 +451,23 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
         confirmPopupPresenter.show("Unsupported DMN asset", event.getMessage());
     }
 
+    @Override
+    public void onEvent(ValidateSimulationEvent event) {
+        scenarioSimulationEditorPresenter.validateSimulation();
+    }
+
     /**
      * Common method to execute the given <code>Command</code> inside the given <code>ScenarioSimulationContext</code>
      * If successful, it adds the command to the <code>ScenarioCommandRegistry</code>, otherwise it fire a new <code>ScenarioNotificationEvent</code>
      * with error details
-     * @param context
      * @param command
+     * @param focusGridAfterExecution
      */
-    protected void commonExecution(final ScenarioSimulationContext context,
-                                   final AbstractScenarioSimulationCommand command,
+    protected void commonExecution(final AbstractScenarioSimulationCommand command,
                                    final boolean focusGridAfterExecution) {
+        final Optional<AbstractScesimGridModel> selectedScenarioGridModel = context.getSelectedScenarioGridModel();
+        final Optional<GridWidget> gridWidget =  selectedScenarioGridModel.isPresent() ? Optional.of(selectedScenarioGridModel.get().getGridWidget()) : Optional.empty();
+        gridWidget.ifPresent(grd -> context.getStatus().setCurrentGrid(grd));
         final CommandResult<ScenarioSimulationViolation> status = scenarioCommandManager.execute(context, command);
         if (Objects.equals(CommandResult.Type.ERROR, status.getType())) {
             String operation = new StringBuilder()
@@ -513,10 +476,10 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
                     .append(" failure")
                     .toString();
             commonNotifyError(status, operation);
-        } else if (Objects.equals(CommandResultBuilder.SUCCESS, status) && command.isUndoable()) {
-            scenarioCommandRegistry.register(context, command);
-            if (focusGridAfterExecution) {
-                context.getScenarioGridPanel().setFocus(true);
+        } else if (Objects.equals(CommandResultBuilder.SUCCESS, status) && (command instanceof AbstractScenarioGridCommand)) {
+            scenarioCommandRegistryManager.register(context, (AbstractScenarioGridCommand) command);
+            if (focusGridAfterExecution && gridWidget.isPresent()) {
+                context.getScenarioGridPanelByGridWidget(gridWidget.get()).setFocus(true);
             }
         }
     }
@@ -558,5 +521,6 @@ public class ScenarioSimulationEventHandler implements AppendColumnEventHandler,
         handlerRegistrationList.add(eventBus.addHandler(SetPropertyHeaderEvent.TYPE, this));
         handlerRegistrationList.add(eventBus.addHandler(UndoEvent.TYPE, this));
         handlerRegistrationList.add(eventBus.addHandler(UnsupportedDMNEvent.TYPE, this));
+        handlerRegistrationList.add(eventBus.addHandler(ValidateSimulationEvent.TYPE, this));
     }
-}
+}
\ No newline at end of file
